from PyQt4 import QtCore, QtGui
import sys
import os.path

from centerBar import Ui_centerBar


from insertDialogWindow import insertDialogWindow

from ActionManagement import ActionManagementDialog
from PropertyManagement import PropertyManagementDialog
from DialogReader import DialogReader
from HistoryDialog import HistoryDialog
from Ui_Editor3 import Ui_Editor3Main
from Editor3CentralWidget import Editor3CentralWidget



class Editor3(QtGui.QMainWindow,Ui_Editor3Main):
    
    def __init__(self,engine):
        QtGui.QMainWindow.__init__(self)
        self.setupUi(self)
        self.engine = engine
        
              
        #set functions
        
        #File
        self.connect(self.FileNew,QtCore.SIGNAL('triggered()'),self.engine.new)
        self.connect(self.FileOpen,QtCore.SIGNAL('triggered()'),self.engine.open)
        self.connect(self.FileSave,QtCore.SIGNAL('triggered()'),self.engine.save)
        self.connect(self.FileSave_As,QtCore.SIGNAL('triggered()'),self.engine.saveAs)
        self.connect(self.FileQuit,QtCore.SIGNAL('triggered()'),QtCore.SLOT('close()'))
        
        #Sentence
        self.connect(self.SentenceAddRoot,QtCore.SIGNAL('triggered()'),self.engine.addNewRoot)
        self.connect(self.SentenceAddSentence,QtCore.SIGNAL('triggered()'),self.engine.addNewSentence)
        self.connect(self.SentenceRemoveOrphans,QtCore.SIGNAL('triggered()'),self.engine.removeOrphans)
        self.connect(self.SentenceRemoveSelected,QtCore.SIGNAL('triggered()'),self.engine.removeSelectedSentence)
        
        #Managers
        self.connect(self.ManagerAction_Manager,QtCore.SIGNAL('triggered()'),self.openActionManager)
        self.connect(self.ManagerLink_Manager,QtCore.SIGNAL('triggered()'),self.openLinkManager)
        self.connect(self.ManagerProperty_Manager,QtCore.SIGNAL('triggered()'),self.openPropertyManager)
        self.connect(self.ManagerSentence_Manager,QtCore.SIGNAL('triggered()'),self.openSentenceManager)
        
        #View
        self.connect(self.ViewHistory,QtCore.SIGNAL('triggered()'),self.openHistoryView)
        self.connect(self.ViewRoots,QtCore.SIGNAL('triggered()'),self.engine.showRoots)
        self.connect(self.ViewStoryline,QtCore.SIGNAL('triggered()'),self.openViewStoryline)
        
        #about
        self.connect(self.AboutAbout,QtCore.SIGNAL('triggered()'),self.openAbout)
        
        #set variables
        self.actionManager = None
        self.propertyManager = None
        self.storylineViewer = None
        self.historyViewer = None
                
    def openActionManager(self):
        self.actionManager = ActionManagementDialog(self,self.engine)
        self.actionManager.exec_()
    
    def openPropertyManager(self):
        self.propertyManager = PropertyManagementDialog(self,self.engine)
        self.propertyManager.exec_()
    
    def openSentenceManager(self):
        pass
    
    def openLinkManager(self):
        pass
    
    def openHistoryView(self):
        self.historyViewer = HistoryDialog(self,self.engine)
        self.historyViewer.show()
    
    def openViewStoryline(self):
        self.storylineViewer = DialogReader(self,self.engine)
        self.storylineViewer.exec_()
    
    def closeEvent(self,event):
        self.engine.saveActiveSentence()
        reply = QtGui.QMessageBox.question(self, 'Exit',"Are you sure you want to quit?", QtGui.QMessageBox.Yes, QtGui.QMessageBox.No)

        if reply == QtGui.QMessageBox.Yes:
            event.accept()
            self.quit()
        else:
            event.ignore()
        
            
    def openAbout(self):
        pass
             
    def quit(self):
        self.showStatusMessage("Closing program")
        if not self.actionManager ==  None:
            self.actionManager.quit()
        if not self.propertyManager == None:
            self.propertyManager.quit()
        if not self.historyViewer == None:
            self.historyViewer.quit()
        if not self.storylineViewer == None:
            self.storylineViewer.quit()    
        
        
    
    def showStatusMessage(self,message):
        self.statusbar.showMessage(message) 

