from PyQt4 import QtCore, QtGui
from Editor3 import Editor3
from Editor3CentralWidget import Editor3CentralWidget
from Database import Database
from exSentence import exSentence
from Sentence import Sentence
from Property import Property
from Link import Link
from exLink import exLink
from Action import Action
from historyHolder import historyHolder
from historyPoint import historyPoint 
from insertDialogWindow import insertDialogWindow
import sys
import os.path      

class ActorList():
    
    def __init__(self):
        self.colorList = [(QtGui.QColor(236,88,0),"Light Red"),(QtGui.QColor(155,221,255),"Light Blue"),(QtGui.QColor(251,236,93),"Light Yellow"),(QtGui.QColor(173,255,47),"Light Green"),(QtGui.QColor(237,28,36),"Red"),(QtGui.QColor(70,130,180),"Blue"),(QtGui.QColor(46,139,87),"Green"),(QtGui.QColor(255,216,0),"Yellow")]
        self.noneColor = QtGui.QColor(255,255,255)
        self.clear()
        
    def addActor(self,actor):
        if not actor == None and len(actor) > 0:
            trimActor = actor.strip()
            upperActor = trimActor.upper()
            
            try:
                self.colorMapping[upperActor]
                return False
            except:
                self.colorMapping[upperActor] = self.colorList[len(self.actorList)]
                self.actorList.append(actor)
                return True
    
    def getActorList(self):
        return self.actorList
    
    def setActorColor(self,actor,qColor):
        trimActor = actor.strip()
        upperActor = trimActor.upper()
        self.colorMapping[upperActor] = (qColor,"user defined")
    
    def count(self):
        return self.actorList.count()
    
    def getColorTupleList(self):
        return self.colorList
    
    def getActorColor(self,actor):
        if not actor == None and len(actor) > 0:
            trimActor = actor.strip()
            upperActor = trimActor.upper()
            try:
                color = self.colorMapping[upperActor]
                if color == None:
                    return self.noneColor
                else:
                    return color[0] 
            except:
                return self.noneColor
        else:
            return self.noneColor  
    
    def clear(self):
        self.actorList = []
        self.colorMapping = dict() 
         
class Editor3Engine():
    
    def __init__(self):
        
        self.mainWindow = Editor3(self)
        self.mainWidget = Editor3CentralWidget(self.mainWindow,self)
        self.mainWindow.setCentralWidget(self.mainWidget)  
                
        self.history = historyHolder()
        self.actorList = ActorList()
        self.database = Database(False)
                
        self.activeDatabaseFilename = None
        
        self.resetAllVariables()
         
        
        QtGui.QApplication.setStyle(QtGui.QStyleFactory.create('windowsxp'))
        
        self.mainWindow.show()
        
        #TODO remove line
        #self.openDatabase("T:/Master Project/test database/test.db3")
        

    def getDatabase(self):
        return self.database
    
    def openDatabase(self,filename):
        filename = str(filename)
        if os.path.isfile(filename):
            self.database.openDatabase(filename)
            self.activeDatabaseFilename = filename
            self.mainWindow.setWindowTitle("Editor3 - "+filename)
            self.loadDatabase()
        else:
            QtGui.QMessageBox.warning(self.mainWindow,"Warning","File was not found: "+str(filename))
    
    def closeDatabase(self):
        self.database.closeDatabase() 
        self.activeDatabaseFilename = None
        self.mainWindow.setWindowTitle("Editor3")
        self.resetAllVariables()
    
    def newDatabase(self,filename):
        filename = str(filename)
        if not os.path.isfile(filename):
            self.database.createNewDatabase(filename)
            self.activeDatabaseFilename = filename
            self.mainWindow.setWindowTitle("Editor3 - "+filename)
            self.loadDatabase()
        else:
            QtGui.QMessageBox.warning(self.mainWindow,"Warning","File already exist try opening the database: "+str(filename))
        
    def resetAllVariables(self):
        
        self.activeSentenceIndex = -1
        self.viewSentences = []
        self.exSentences = []
        
        self.history.clearAll()
        self.actorList.clear()
        
        self.mainWidget.clean()
        
        self.mainWindow.MenuKind.clear()
   
    def getActorList(self):
        return self.actorList
    
    def open(self):
        #print self.activeDatabaseFilename
        if self.activeDatabaseFilename == None:
            #open
            filename = QtGui.QFileDialog.getOpenFileName(self.mainWindow, "Select db3", "*.db3")

            if not filename == None and len(filename) > 0 and os.path.isfile(filename):
                
                self.openDatabase(filename)
            else:
                pass
        else:
            self.saveActiveSentence()
            
            self.closeDatabase()
            self.open()
                
    def saveAs(self):
        filename = str(QtGui.QFileDialog.getSaveFileName(self.mainWindow, "Select location and filename", "*.db3"))
        if not filename == None and len(filename) > 0:
            
            if not os.path.isfile(filename):
                self.database.backUpDatabase(filename, False)
                self.database.closeDatabase()
                self.database.openDatabase(filename)
                self.saveActiveSentence()

                self.setWindowTitle("Editor3 - "+filename)
            else:
                reply = QtGui.QMessageBox.question(self, 'Save as',"Are you sure you want to overwrite this file: "+str(filename)+"?", QtGui.QMessageBox.Yes, QtGui.QMessageBox.No)
    
                if reply == QtGui.QMessageBox.Yes:
                    self.database.backUpDatabase(filename, True)
                    self.database.closeDatabase()
                    self.database.openDatabase(filename)
                    self.saveActiveSentence()
                    self.setWindowTitle("Editor3 - "+filename)
                else:
                    self.saveAs()
        else: 
            pass
    
    def save(self):
        if self.activeDatabaseFilename == None:
            self.saveAs()
        else:
            self.saveActiveSentence()
    
    def new(self):
        if self.activeDatabaseFilename == None:
            filename = QtGui.QFileDialog.getSaveFileName(self.mainWindow, "Select location and filename", "*.db3")
            #print filename
            if not filename == None and len(filename) > 0:
                
                self.newDatabase(filename)
            else:
                pass
        else:
            self.saveActiveSentence()
            self.closeDatabase()
            self.new()
                
    def saveActiveSentence(self):
        if not self.getActiveSentence() == None:
            
            self.saveSentence(self.activeSentenceIndex)
            
    
    def loadDatabase(self):
        #print "load database"
        self.resetAllVariables()
        
        #load all actors
        self.updateActorList()
        try:
            pId = self.database.getPropertyId('avatar_type')
            self.database.showProperty(pId, False)
            pId = self.database.getPropertyId('is_root')
            self.database.showProperty(pId, False)
            pId = self.database.getPropertyId('audio_url')
            self.database.showProperty(pId, True)
            pId = self.database.getPropertyId('is_empty')
            if pId == None:
                self.database.addNewProperty('is_empty', False)
            else:
                self.database.showProperty(pId,False)
        except:
            print "malformed database"        
        
        #load roots
        roots = self.getRootSentences(True,False)
        
        self.exSentences = roots
        self.updateView()
    
    def jumpToHistory(self,historyPoint):
        self.history.jumpToHistoryPoint(historyPoint)
        point = self.history.getCurrentHistoryPoint()
        sentences = []
        activeSentence = None
        for id in point.getViewSentenceIds():
            sentence = self.getSentence(id)
            sentences.append(sentence)
            if id == point.getActiveSentenceId():
                activeSentence = sentence
                
        
               
        self.exSentences = sentences
        self.setActiveSentence(sentences.index(activeSentence))
        #self.updateView()
        
    def updateView(self):
        
        self.mainWidget.clean()
        self.updateActorList()
        #check if there is a previous sentence
        previousSentence = self.history.showPreviousHistoryPoint()
        prev = None
        
        if not previousSentence == None:
            
            prev = self.getSentence(previousSentence.getActiveSentenceId())
                
        #check if there is a next sentence    
        nextSentence = self.history.showNextHistoryPoint()
        next = None
        if not nextSentence == None:
            next = self.getSentence(nextSentence.getActiveSentenceId())
                    
        counter = 0
        while counter < len(self.exSentences):
            
            self.exSentences[counter] = self.getSentence(self.exSentences[counter].getId())
            counter += 1
            
        self.viewSentences = self.mainWidget.setView(prev,self.exSentences,next)
        
        
        
    def updateScreen(self):
        if self.activeSentenceIndex >= 0 and self.activeSentenceIndex < len(self.viewSentences): 
            self.viewSentences[self.activeSentenceIndex].save()
            self.mainWidget.saveProperties()
        self.viewSentences = self.mainWidget.updateScreen()    
        
    def updateActorList(self):
        actors = self.database.getAllPropertyValues("avatar_type")
        
        for actor in actors:
           if self.actorList.addActor(actor):
                
                menuKind_1 = QtGui.QMenu(self.mainWindow.MenuKind)
                menuKind_1.setObjectName(actor)
                menuKind_1.setTitle(actor)
                
                for color in self.actorList.getColorTupleList():
                    actionColor = QtGui.QAction(menuKind_1)
                    actionColor.setObjectName("actionColor")
                    actionColor.setText(color[1])
                    self.mainWindow.connect(actionColor,QtCore.SIGNAL('triggered()'),self.createColorSwitch(actor,color[0]))
                    menuKind_1.addAction(actionColor)
                    
                    #self.mainWindow.MenuKind.addAction(menuKind_1.menuAction())
                    
                actionColor = QtGui.QAction(menuKind_1)
                actionColor.setObjectName("actionColor")
                actionColor.setText("Choose")
                self.mainWindow.connect(actionColor,QtCore.SIGNAL('triggered()'),self.createColorSelect(actor))
                menuKind_1.addAction(actionColor)
                
                self.mainWindow.MenuKind.addAction(menuKind_1.menuAction())
        
    def createColorSwitch(self,actor,color):
        def function():
            self.actorList.setActorColor(actor,color)
            self.updateScreen() 
        return function
    
    def createColorSelect(self,actor):
        def function():
            color = QtGui.QColorDialog.getColor()
            if not color == None:
                self.actorList.setActorColor(actor,color)
                self.updateScreen()
        return function
    
    def getSentence(self,sentenceId):
        sentence = self.database.getSentenceById(int(sentenceId), True, False)
        exS = None
        if not sentence == None:
            exS = exSentence(sentence,self)
        else:
            print "Didn't find sentence with id: "+str(sentenceId)
        return exS
    
    def getRootSentences(self,peek=False,shownOnly=False):
        roots = self.database.getRootSentences(peek, shownOnly)
        exRoots = []
        for root in roots:
            exRoots.append(exSentence(root,self))
        
        return exRoots
    
    def getNewSentence(self):
        newSentence = Sentence("Empty Sentence")
        property = Property("is_empty","1")
        newSentence.addProperty(property)
        
        newSentence = self.database.updateSentence(newSentence, True, False)
        
        return exSentence(newSentence,self)
    
    def updateSentence(self,index):
        if index < len(self.exSentences) and index >= 0:
            self.exSentences[index] = exSentence(self.database.updateSentence(self.exSentences[index], True, False),self)
            
            self.updateActorList()
                
        
    def addNewRoot(self):
        newRoot = self.getNewSentence()
        #print newRoot.getProperty("is_root")
        property = Property("is_root",self.database.ENABLED)
        newRoot.addProperty(property)
        
        
        self.exSentences.append(newRoot)
        self.updateSentence(len(self.exSentences)-1)
        self.setActiveSentence(len(self.exSentences)-1)
        
    
    def addNewSentence(self):
        newSentence = self.getNewSentence()
        self.exSentences.append(newSentence)
        self.setActiveSentence(len(self.exSentences)-1)
    
    
    def removeSelectedSentence(self):
        try:
            
            self.removeSentence(self.activeSentenceIndex)
        
        except:
            pass
        
        
    def addNewLink(self,index):
        newSentence = self.getNewSentence()
        sentence = self.exSentences[index]
        link = Link(sentence.getId(),newSentence.getId(),len(sentence.getChildren()))
        
        xLink = exLink(link,sentence.getActor(),newSentence.getActor(),self)
        
        sentence.addChild(xLink)
        
        self.updateLinks(sentence)
                            
        self.updateScreen()
    
    def addExistingLink(self,index):
        itemList = []
        sentenceList = self.database.getAllSentences(False,True)
        sentence = self.exSentences[index]
        for sen in sentenceList:
            itemList.append(str(sen.getSentence())+" ["+str(sen.getId())+"]")
        
        insertDialog = insertDialogWindow(self.mainWindow,"Existing Sentence","Search:","select",itemList,True,None)
        insertDialog.exec_()
        result = insertDialog.getResult()
        
        if not result == None:
            rIndex = itemList.index(result)
            childId = sentenceList[rIndex].getId()
            childSentence = exSentence(sentenceList[rIndex],self)
            #print "link: "+str(sentence.getId())+" "+str(childId)
            link = Link(sentence.getId(),childId,len(sentence.getChildren()))
            xLink = exLink(link,sentence.getActor(),childSentence.getActor(),self)
            sentence.addChild(xLink)
            self.updateLinks(sentence)
        self.updateScreen()
        
    def linkUp(self,parentIndex,childId):
        sentence = self.exSentences[parentIndex]
        
        childs = sentence.getChildrenOrdered()
        counter = 1
        while(counter < len(childs)):
            if childs[counter].getReplyId() == childId:
                if counter-1 < len(childs):
                    swapOrder = childs[counter].getOrder()
                    childs[counter].setOrder(childs[counter-1].getOrder())
                    childs[counter-1].setOrder(swapOrder)
                    if swapOrder == childs[counter].getOrder():
                                childs[counter].setOrder(childs[counter].getOrder()+1)                         
            counter += 1
            self.updateLinks(sentence)
               
        self.updateScreen()
    
    def linkDown(self,parentIndex, childId):
        #print "up "+str(parentId)+" "+str(childId)
        
        sentence = self.exSentences[parentIndex]
        
        childs = sentence.getChildrenOrdered()
        counter = 0
        while(counter < len(childs)):
            if childs[counter].getReplyId() == childId:
                if counter+1 < len(childs):
                    swapOrder = childs[counter].getOrder()
                    childs[counter].setOrder(childs[counter+1].getOrder())
                    childs[counter+1].setOrder(swapOrder)
                    if swapOrder == childs[counter].getOrder():
                                childs[counter].setOrder(childs[counter].getOrder()-1)
            counter += 1
            self.updateLinks(sentence)
                
        self.updateScreen()
    
    def linkRemove(self,parentIndex,childId):
        parentSentence = self.exSentences[parentIndex]
        parentId = parentSentence.getId()
        childSentence = self.getSentence(childId)
        message = QtGui.QMessageBox(self.mainWindow)
        message.setText("Are you sure you want to remove the link between: \n "+str(parentSentence.getSentence())+" \n "+str(childSentence.getSentence()))
        linkOnly = message.addButton("Link Only",QtGui.QMessageBox.ActionRole)
        linkOrphans = message.addButton("Link + Orphans",QtGui.QMessageBox.ActionRole)
        
        linkCancel = message.addButton("Cancel",QtGui.QMessageBox.RejectRole)
        
        message.exec_()
        
        if message.clickedButton() == linkCancel:
            #do nothing
            pass
        elif message.clickedButton() == linkOnly:
            #remove link
            self.database.removeChildFrom(parentId, childId)
            parentSentence.removeChild(childId)
            self.updateScreen()
        elif message.clickedButton() == linkOrphans:
            #remove link and orphans
            self.database.removeChildFrom(parentId, childId)
            childSentence = self.getSentence(childId)
            if len(childSentence.getParents()) == 0:
                self.database.removeSentenceWithoutFloat(childId)
                
            
            parentSentence.removeChild(childId)
              
            self.updateScreen()
         
            
    def updateLinks(self,sentence):
        children = sentence.getChildren()
        for child in children:
            self.database.updateLink(child)
    
    def sentenceClicked(self,sentenceId,parentId):
        
        self.saveActiveSentence() 
        
        if parentId == None:
            newActiveSentence = self.history.getPreviousHistoryPoint()
            self.exSentences = []
            sentence = self.getSentence(newActiveSentence.getActiveSentenceId())
            
            newPreviousSentence = self.history.showPreviousHistoryPoint()
            
            
            if not newPreviousSentence == None:
                parentSentence = self.getSentence(newPreviousSentence.getActiveSentenceId())
                for child in parentSentence.getChildrenOrdered():
                    childSentence = self.getSentence(child.getReplyId())
                             
                    self.exSentences.append(childSentence)
            else:
                
                self.exSentences.append(sentence)
                   
            self.setActiveSentence(self.exSentences.index(sentence))
            
            
        else:
            #move forward
            
            parentSentence = exSentence(self.database.getSentenceById(int(parentId), False, True),self)
            
            #check if moving over history
            nextSentence = self.history.showNextHistoryPoint()
           
            activeSentence = None
                           
            if not nextSentence == None and nextSentence.getActiveSentenceId() == sentenceId:
                #moving over history
                nextSentence = self.history.getNextHistoryPoint()
                self.exSentences = []
                for child in parentSentence.getChildrenOrdered():
                    sentence = self.getSentence(child.getReplyId())
                    if sentence.getId() == sentenceId:
                        activeSentence = sentence
                    
                    self.exSentences.append(sentence)
            else:
                #new history point
                viewSentencesIds = []
                for sentence in self.exSentences:
                    viewSentencesIds.append(sentence.getId())
                    
                self.exSentences = []
                
                for child in parentSentence.getChildrenOrdered():
                    sentence = self.getSentence(child.getReplyId())
                    if sentence.getId() == sentenceId:
                        activeSentence = sentence
                    
                    
                    self.exSentences.append(sentence)
                    
                self.history.addToHistory(historyPoint(parentId,viewSentencesIds))
            
            self.setActiveSentence(self.exSentences.index(activeSentence))
        self.updateView()
           
                
    def createNewLinkFunction(self,sentenceIndex):
        def linker(): 
            self.addNewLink(sentenceIndex) 
        
        return linker
    
    def createExistingLinkFunction(self,sentenceIndex):
        def linker():
            self.addExistingLink(sentenceIndex)
        
        return linker
    
    def createSentenceClicked(self,sentenceId,parentId):
        def function():
            self.sentenceClicked(sentenceId,parentId)
        
        return function
    
    def createNowSentenceClicked(self,sentenceIndex):
        def function():
            self.setActiveSentence(sentenceIndex)
            
        return function
    
    def createLinkUpFunction(self,parentIndex,childId):
        def function():
            self.linkUp(parentIndex,childId)
            
        return function
    
    def createLinkDownFunction(self,parentIndex,childId):
        def function():
            self.linkDown(parentIndex,childId)
            
        return function
    
    def createLinkRemoveFunction(self,parentIndex,childId):
        def function():
            self.linkRemove(parentIndex,childId)
        
        return function
    
    def setActiveSentence(self,sentenceIndex):
        self.saveActiveSentence()
        self.activeSentenceIndex = sentenceIndex
        self.updateView()        
    
    def getActiveSentence(self):
        if len(self.viewSentences) > self.activeSentenceIndex and self.activeSentenceIndex >= 0:
            return self.viewSentences[self.activeSentenceIndex]
        else:
            return None
    
    def getActiveSentenceId(self):
        if len(self.exSentences) > self.activeSentenceIndex and self.activeSentenceIndex >= 0:
            return self.exSentences[self.activeSentenceIndex].getId()
        else:
            return -1
    
    def saveSentence(self,sentenceIndex):
        try:
            self.viewSentences[sentenceIndex].save()
            self.mainWidget.saveProperties()
            self.updateSentence(sentenceIndex)
                        
        except ValueError:
            print "sentence could not be found in list: "+str(sentenceIndex)
            
            
            
    def removeSentence(self,sentenceIndex):
        sentence = self.exSentences[sentenceIndex]
        message = QtGui.QMessageBox(self.mainWindow)
        message.setText("Are you sure you want to remove this sentence: \n ["+str(sentence.getId())+"] "+str(sentence.getSentence()))
        sentenceOnly = message.addButton("Sentence Only",QtGui.QMessageBox.ActionRole)
        sentenceOrphans = message.addButton("Sentence + Orphans",QtGui.QMessageBox.ActionRole)
        
        sentenceCancel = message.addButton("Cancel",QtGui.QMessageBox.RejectRole)
        
        message.exec_()
        
        if message.clickedButton() == sentenceCancel:
            #do nothing
            pass
        elif message.clickedButton() == sentenceOnly:
            self.database.removeSentence(sentence.getId())
            self.exSentences.remove(sentence)
            self.setActiveSentence(-1)
            self.updateView()
        elif message.clickedButton() == sentenceOrphans:
            self.database.removeSentenceWithoutFloat(sentence.getId())
            self.exSentences.remove(sentence)
            self.setActiveSentence(None) 
            self.updateView() 
            
    def createRevertBackSentenceFunction(self,sentenceIndex):
        def revert():
            self.exSentences[sentenceIndex] = exSentence(self.database.getSentenceById(self.exSentences[sentenceIndex].getId(),True,False),self)
            self.updateView()
            
        return revert
    
    def createRemoveSentenceFunction(self,sentenceIndex):
        
        def remove():
            
            self.removeSentence(sentenceIndex)
        return remove
    
    def addNewProperty(self):
        
        if not self.activeSentenceIndex == -1:
            activeSentence = self.exSentences[self.activeSentenceIndex]
            
            dialog = insertDialogWindow(self.mainWindow,"Add new option","New:","Add",activeSentence.getPropertys(),False,None)
            dialog.exec_()
            
            result = dialog.getResult()
           
            if not result == None:
                newProperty = Property(result,"default value")
                activeSentence.addProperty(newProperty)
                #self.db.updatePropertyList(newProperty,activeSentence.getId())
                self.updateScreen()
                
    def addExistingProperty(self):
        if not self.activeSentenceIndex == -1:
            activeSentence = self.exSentences[self.activeSentenceIndex]
            
            propList = self.database.getAllProperties(False)
            #print propList
            for property in activeSentence.getPropertys():
                propList.remove(property)
            
            dialog = insertDialogWindow(self.mainWindow,"Select existing option","Search:","Add",propList,True,None)
            dialog.exec_()
            
            result = dialog.getResult()
            
            if not result == None:
                newProperty = Property(result,"default Value",self.database.getPropertyId(result))
                activeSentence.addProperty(newProperty)
                #self.db.updatePropertyList(newProperty,activeSentence.getId())
                self.updateScreen()
                
    def getAllActions(self):
        return self.database.getAllActions()                    
    
    def removeAction(self,sentenceIndex,actionId,actionKind):
        sentence = self.exSentences[sentenceIndex]
        sentenceId = sentence.getId()
        
        #reorder
              
        list = []
        if actionKind == Action.PREACTION:
            list = sentence.getPreActionList()
        else:
            list = sentence.getPostActionList()
        actionOrder = -1
        removeAction = None    
        for action in list:
            if actionOrder == -1 and action.getId() == actionId:
                actionOrder = action.getOrder()
                removeAction = action
            if not actionOrder == -1:
                swap = action.getOrder()
                if not swap == -1:
                    action.setOrder(actionOrder)
                    actionOrder = swap
            
        del list[list.index(removeAction)]
        
        self.database.removeActionFrom(sentenceId, actionId, actionKind)
        
        self.updateScreen()
    
    def setHistoryWidget(self,widget):
        
        self.history.setDialog(widget)        
    
    def historyWindowClosed(self):
        self.history.setDialog(None)
    
    def removeOrphans(self):
        self.database.removeAllOrphans()
        
    def showRoots(self):
        self.saveActiveSentence()
        self.resetAllVariables()
        self.exSentences = self.getRootSentences(True, False)
        self.updateView()
        
    def quit(self):
        self.mainWindow.quit()
        
        
