from Action import Action
from Link import Link
from Property import Property

class Sentence:
    #Pre: sentence is a String, id is a integer
    #Post: Created Sentence object
    #comment: Can create empty sentence or give the sentence String and or id number 
    #     Id number should only be set by the database setting a random number may 
    #     create weird unpredictable effects
    def __init__(self,sentence="",id=None):
        self.id=id
        self.sentence = sentence
        
        self.propertyList = dict()
        self.preActions = []
        self.postActions = []
        
        self.children = []
        self.parents = []
    
    #Pre: 
    #Post: returns a integer id number
    #comment: If no id is set returns None    
    def getId(self):
        return self.id
    
    #Pre:
    #Post: returns a list with Link objects that are children of this sentence
    #comment: returns empty list if no children    
    def getChildren(self):
        return self.children
    
    #Pre:
    #Post: returns a ordered list with Link object that are children of this sentence
    #comment: returns a empty list if no children
    #         If no order was set the Link should end up at the begining of the list
    #         Object with the same order can end up in any particular order from each other
    #         Order is ascending
    def getChildrenOrdered(self):
        self.children.sort(cmp=self.ordering)
        return self.children
    
    #Pre:
    #Post: returns a list with Link objects that are parents of this sentence
    #comment: returns empty list if no parents
    def getParents(self):
        return self.parents
    
    #Pre: link is a Link object of a existing link in the database
    #Post: Adds a child link to the END of the children list
    #comment: Should only be set by the database
    
    def addChild(self,link):
        self.children.append(link)
    
    #Pre: list is a list of link objects representing existing links in the database
    #Post: Adds all child links in same order to the END of the children list
    #comment: Should only be set by the database
    #         Order in the list does not have to represent the order of the childeren
    def addChildren(self,list):
        for item in list:
            self.addChild(item)
    
    #Pre: link is a Link Object of a existing link in the database
    #Post: Adds a parent link to the END of the parents list
    #comment: Should only be set by the database
    #         The item order in the list represents no specific ordering of parent sentences
    #         the order number in the Link object has no meaning for child to parent only from parent to child
    def addParent(self,link):
        self.parents.append(link)
    
    #Pre: list is a list of Link objects representing existing links in the database
    #Post: Adds all parent links to the parent list
    #comment: Should only be set by the database
    #         Also see addParent
    def addParents(self,list):
        for item in list:
            self.addParent(item)
    
    #Pre: sentence is a String
    #Post: sets the sentence string of the sentence object
    #comment: Does not change the sentence String in the database with the same Id
    #         Use updateSentence method of the database to execute changes
    def setSentence(self,sentence):
        self.sentence = sentence
    
    #Pre:
    #Post: Returns the sentence string
    #comment: Returns "" by default if no sentence string was set
    def getSentence(self):
        return self.sentence
    
    #Pre: name is a existing property name in the database linked to this sentence
    #Post: returns the property object
    #comment: If no value was set or name is invalid None is returned    
    def getProperty(self,name):
        try:
            return self.propertyList[name]
        except:
            return None
    
    #Pre: property is a Property Object
    #Post: sets the value of property name
    #comment: Does not change the property value in the database with the same Id
    #         Use updateSentence method of the database to execute changes
    #         If the property didn't yet exist in the database it will be automatically added
    #         with show ENABLED
    def addProperty(self,property):
        if len(property.getName()) > 0:
            self.propertyList[property.getName()] = property
    
    #Pre:
    #Post: returns list of all property names of properties linked to this sentences and 
    #      show value of the property on ENABLED
    #comment:    
    def getPropertys(self):
        return self.propertyList.keys()
    
    #Pre:
    #Post: returns a Dict with all property names and values linked to this sentence and
    #      show value of the property's on ENABLED
    #comment: Adding or changing values in the returned dictionairy should have no effect
    #         on the property list of this sentence even after using sentenceUpdate from
    #         the database
    def getPropertyDict(self):
        return self.propertyList
    
    #Pre:
    #Post: returns a list with all preActions linked to this sentence
    #comment: Returns a empty list if no preActions are set
    #         The Action object order in the list does not have to represent the order
    #         of the preActions
    def getPreActionList(self):
        return self.preActions
    
    #Pre:
    #Post: returns a ordered list with all preActions linked to this sentence
    #comment: Returns a empty list if no preActions are set
    #         If no order was set the Action should end up at the begining of the list
    #         Object with the same order can end up in any particular order from each other
    #         Order is ascending
    def getPreActionListOrdered(self):
        self.preActions.sort(cmp=self.ordering)
        return self.preActions
    #Pre: index is in range of the preAction list
    #Post: returns the preAction in the list on index
    #comment: If index is invalid None is returned
    #         The Action object order in the list does not have to represent the order
    #         of the preActions
    def getPreAction(self,index):
        try:
            return self.preActions[index]
        except:
            return None
    
    #Pre: action is a Action object  with kind PREACTION
    #Post: adds the action object to the END of the preAction list
    #comment:
    def addPreAction(self,action):
        if not action == None and action.getKind() == Action.PREACTION :
            self.preActions.append(action)
    
    #Pre:
    #Post: returns a list with all postActions linked to this sentence
    #comment: Returns a empty list if no postActions are set
    #         The Action object order in the list does not have to represent the order
    #         of the postActions                
    def getPostActionList(self):
        return self.postActions
    
    #Pre:
    #Post: returns a ordered list with all postActions linked to this sentence
    #comment: Returns a empty list if no postActions are set
    #         If no order was set the Action should end up at the begining of the list
    #         Object with the same order can end up in any particular order from each other
    #         Order is ascending
    def getPostActionListOrdered(self):
        self.postActions.sort(cmp=self.ordering)
        return self.postActions
    #Pre: index is in range of the postAction list
    #Post: returns the postAction in the list on index
    #comment: If index is invalid None is returned
    #         The Action object order in the list does not have to represent the order
    #         of the postActions
    def getPostAction(self,index):
        try:
            return self.postActions[index]
        except:
            return None
    
    #Pre: action is a Action object with kind POSTACTION
    #Post: adds the action object to the END of the postAction list
    #comment:
    def addPostAction(self,action):
        if not action == None and action.getKind() == Action.POSTACTION:
            self.postActions.append(action)
    
    #Pre: 
    #Post: Returns True if this sentence doesn't have any children
    #comment: If between the creation of this object and this method call a link
    #         is added to the database the returned value of this method could be incorrect
    def isLeafnode(self):
        return len(self.children) == 0
    
    #Pre:
    #Post: Returns True if this sentence has a property is_root and the property is set to 1
    #comment:
    def isRootNode(self):
        prop = self.getProperty("is_root")
        if prop == None:
            return False
        else:
            if int(prop.getValue()) == 1:
                return True
            else:
                return False
    
    #To String    
    def __str__(self):
        return "("+str(self.id)+") "+self.sentence
    
    #Pre: x and y are object with getOrder method
    #Post: returns 1 if x > y, -1 if x < y, 0 otherwise
    #comment: Method should only be used for sort(ordering) function
    def ordering(self,x,y):
        try:
            if x.getOrder() > y.getOrder():
                return 1
            elif x.getOrder() < y.getOrder():
                return -1
            else:
                return 0
        except:
            return 0
            